@@ -428,19 +428,40 @@ nc_session_rpc_unlock(struct nc_session *session, int timeout, const char *func)
 }
 
 int
-nc_session_io_lock(struct nc_session *session, int timeout, const char *func)
+nc_rwlock_lock(pthread_rwlock_t *rwlock, enum nc_rwlock_mode mode, int timeout, const char *func_name)
 {
     int ret;
     struct timespec ts_timeout;
 
+    if (!rwlock || (mode == NC_RWLOCK_NONE)) {
+        ERRINT;
+        return -1;
+    }
+
     if (timeout > 0) {
+        /* get absolute time for timeout */
         nc_timeouttime_get(&ts_timeout, timeout);
 
-        ret = pthread_mutex_clocklock(session->io_lock, COMPAT_CLOCK_ID, &ts_timeout);
+        /* acquire lock with timeout based on mode */
+        if (mode == NC_RWLOCK_READ) {
+            ret = pthread_rwlock_clockrdlock(rwlock, COMPAT_CLOCK_ID, &ts_timeout);
+        } else {
+            ret = pthread_rwlock_clockwrlock(rwlock, COMPAT_CLOCK_ID, &ts_timeout);
+        }
     } else if (!timeout) {
-        ret = pthread_mutex_trylock(session->io_lock);
-    } else { /* timeout == -1 */
-        ret = pthread_mutex_lock(session->io_lock);
+        /* try to acquire lock without waiting */
+        if (mode == NC_RWLOCK_READ) {
+            ret = pthread_rwlock_tryrdlock(rwlock);
+        } else {
+            ret = pthread_rwlock_trywrlock(rwlock);
+        }
+    } else {
+        /* acquire lock without timeout */
+        if (mode == NC_RWLOCK_READ) {
+            ret = pthread_rwlock_rdlock(rwlock);
+        } else {
+            ret = pthread_rwlock_wrlock(rwlock);
+        }
     }
 
     if (ret) {
@@ -449,54 +470,53 @@ nc_session_io_lock(struct nc_session *session, int timeout, const char *func)
             return 0;
         }
 
-        /* error */
-        ERR(session, "%s: failed to IO lock a session (%s).", func, strerror(ret));
+        ERR(NULL, "%s: failed to lock rwlock in %s mode (%s).", func_name,
+                mode == NC_RWLOCK_READ ? "read" : "write", strerror(ret));
         return -1;
     }
 
     return 1;
 }
 
-int
-nc_session_io_unlock(struct nc_session *session, const char *func)
+void
+nc_rwlock_unlock(pthread_rwlock_t *rwlock, const char *func_name)
 {
-    int ret;
+    int r;
 
-    ret = pthread_mutex_unlock(session->io_lock);
-    if (ret) {
-        /* error */
-        ERR(session, "%s: failed to IO unlock a session (%s).", func, strerror(ret));
-        return -1;
+    if (!rwlock) {
+        ERRINT;
+        return;
     }
 
-    return 1;
+    r = pthread_rwlock_unlock(rwlock);
+    if (r) {
+        ERR(NULL, "%s: failed to unlock rwlock (%s).", func_name, strerror(r));
+    }
 }
 
 int
-nc_session_client_msgs_lock(struct nc_session *session, int *timeout, const char *func)
+nc_mutex_lock(pthread_mutex_t *mutex, int timeout, const char *func_name)
 {
     int ret;
-    int32_t diff_msec;
-    struct timespec ts_timeout, ts_start;
+    struct timespec ts_timeout;
 
-    assert(session->side == NC_CLIENT);
+    if (!mutex) {
+        ERRINT;
+        return -1;
+    }
 
-    if (*timeout > 0) {
-        /* get current time */
-        nc_timeouttime_get(&ts_start, 0);
+    if (timeout > 0) {
+        /* get absolute time for timeout */
+        nc_timeouttime_get(&ts_timeout, timeout);
 
-        nc_timeouttime_get(&ts_timeout, *timeout);
-
-        ret = pthread_mutex_clocklock(&session->opts.client.msgs_lock, COMPAT_CLOCK_ID, &ts_timeout);
-        if (!ret) {
-            /* update timeout based on what was elapsed */
-            diff_msec = nc_timeouttime_cur_diff(&ts_start);
-            *timeout -= diff_msec;
-        }
-    } else if (!*timeout) {
-        ret = pthread_mutex_trylock(&session->opts.client.msgs_lock);
-    } else { /* timeout == -1 */
-        ret = pthread_mutex_lock(&session->opts.client.msgs_lock);
+        /* acquire lock with timeout */
+        ret = pthread_mutex_clocklock(mutex, COMPAT_CLOCK_ID, &ts_timeout);
+    } else if (!timeout) {
+        /* try to acquire lock without waiting */
+        ret = pthread_mutex_trylock(mutex);
+    } else {
+        /* acquire lock without timeout */
+        ret = pthread_mutex_lock(mutex);
     }
 
     if (ret) {
@@ -505,29 +525,27 @@ nc_session_client_msgs_lock(struct nc_session *session, int *timeout, const char
             return 0;
         }
 
-        /* error */
-        ERR(session, "%s: failed to MSGS lock a session (%s).", func, strerror(ret));
+        ERR(NULL, "%s: failed to lock mutex (%s).", func_name, strerror(ret));
         return -1;
     }
 
     return 1;
 }
 
-int
-nc_session_client_msgs_unlock(struct nc_session *session, const char *func)
+void
+nc_mutex_unlock(pthread_mutex_t *mutex, const char *func_name)
 {
-    int ret;
+    int r;
 
-    assert(session->side == NC_CLIENT);
-
-    ret = pthread_mutex_unlock(&session->opts.client.msgs_lock);
-    if (ret) {
-        /* error */
-        ERR(session, "%s: failed to MSGS unlock a session (%s).", func, strerror(ret));
-        return -1;
+    if (!mutex) {
+        ERRINT;
+        return;
     }
 
-    return 1;
+    r = pthread_mutex_unlock(mutex);
+    if (r) {
+        ERR(NULL, "%s: failed to unlock mutex (%s).", func_name, strerror(r));
+    }
 }
 
 API NC_STATUS
@@ -788,7 +806,7 @@ nc_session_free_transport(struct nc_session *session, int *multisession)
          * it. Also, avoid concurrent free by multiple threads of sessions that share the SSH session.
          */
         /* SESSION IO LOCK */
-        r = nc_session_io_lock(session, NC_SESSION_FREE_LOCK_TIMEOUT, __func__);
+        r = nc_mutex_lock(session->io_lock, NC_SESSION_FREE_LOCK_TIMEOUT, __func__);
 
         if (session->ti.libssh.next) {
             for (siter = session->ti.libssh.next; siter != session; siter = siter->ti.libssh.next) {
@@ -843,7 +861,7 @@ nc_session_free_transport(struct nc_session *session, int *multisession)
 
         /* SESSION IO UNLOCK */
         if (r == 1) {
-            nc_session_io_unlock(session, __func__);
+            nc_mutex_unlock(session->io_lock, __func__);
         }
         break;
     }
@@ -881,7 +899,7 @@ nc_session_free_transport(struct nc_session *session, int *multisession)
 API void
 nc_session_free(struct nc_session *session, void (*data_free)(void *))
 {
-    int r, i, rpc_locked = 0, msgs_locked = 0, timeout;
+    int r, i, rpc_locked = 0, msgs_locked = 0;
     int multisession = 0; /* flag for more NETCONF sessions on a single SSH session */
     struct nc_msg_cont *contiter;
     struct ly_in *msg;
@@ -894,15 +912,18 @@ nc_session_free(struct nc_session *session, void (*data_free)(void *))
     }
 
     if ((session->side == NC_SERVER) && (session->flags & NC_SESSION_CALLHOME)) {
-        /* CH LOCK */
-        pthread_mutex_lock(&session->opts.server.ch_lock);
+        /* CH LOCK, continue on error */
+        r = nc_mutex_lock(&session->opts.server.ch_lock, NC_SESSION_CH_LOCK_TIMEOUT, __func__);
     }
 
     status = session->status;
 
     if ((session->side == NC_SERVER) && (session->flags & NC_SESSION_CALLHOME)) {
         /* CH UNLOCK */
-        pthread_mutex_unlock(&session->opts.server.ch_lock);
+        if (!r) {
+            /* only if we locked it */
+            nc_mutex_unlock(&session->opts.server.ch_lock, __func__);
+        }
     }
 
     if (status == NC_STATUS_CLOSING) {
@@ -945,10 +966,8 @@ nc_session_free(struct nc_session *session, void (*data_free)(void *))
     }
 
     if (session->side == NC_CLIENT) {
-        timeout = NC_SESSION_FREE_LOCK_TIMEOUT;
-
         /* MSGS LOCK */
-        r = nc_session_client_msgs_lock(session, &timeout, __func__);
+        r = nc_mutex_lock(&session->opts.client.msgs_lock, NC_SESSION_FREE_LOCK_TIMEOUT, __func__);
         if (r == -1) {
             return;
         } else if (r) {
@@ -969,7 +988,7 @@ nc_session_free(struct nc_session *session, void (*data_free)(void *))
 
         if (msgs_locked) {
             /* MSGS UNLOCK */
-            nc_session_client_msgs_unlock(session, __func__);
+            nc_mutex_unlock(&session->opts.client.msgs_lock, __func__);
         }
 
         if ((session->status == NC_STATUS_RUNNING) && nc_session_is_connected(session)) {
@@ -997,7 +1016,7 @@ nc_session_free(struct nc_session *session, void (*data_free)(void *))
 
     if ((session->side == NC_SERVER) && (session->flags & NC_SESSION_CALLHOME)) {
         /* CH LOCK */
-        pthread_mutex_lock(&session->opts.server.ch_lock);
+        nc_mutex_lock(&session->opts.server.ch_lock, NC_SESSION_CH_LOCK_TIMEOUT, __func__);
     }
 
     /* mark session for closing */
@@ -1020,7 +1039,7 @@ nc_session_free(struct nc_session *session, void (*data_free)(void *))
 
     if ((session->side == NC_SERVER) && (session->flags & NC_SESSION_CALLHOME)) {
         /* CH UNLOCK */
-        pthread_mutex_unlock(&session->opts.server.ch_lock);
+        nc_mutex_unlock(&session->opts.server.ch_lock, __func__);
     }
 
     /* transport implementation cleanup */
@@ -1156,7 +1175,9 @@ nc_server_get_cpblts_version(const struct ly_ctx *ctx, LYS_VERSION version)
     }
 
     /* HELLO LOCK */
-    pthread_rwlock_rdlock(&server_opts.hello_lock);
+    if (nc_rwlock_lock(&server_opts.hello_lock, NC_RWLOCK_READ, NC_HELLO_LOCK_TIMEOUT, __func__) != 1) {
+        goto error;
+    }
 
     mod = ly_ctx_get_module_implemented(ctx, "ietf-netconf-with-defaults");
     if (mod) {
@@ -1300,7 +1321,7 @@ nc_server_get_cpblts_version(const struct ly_ctx *ctx, LYS_VERSION version)
     }
 
     /* HELLO UNLOCK */
-    pthread_rwlock_unlock(&server_opts.hello_lock);
+    nc_rwlock_unlock(&server_opts.hello_lock, __func__);
 
     /* ending NULL capability */
     add_cpblt(NULL, &cpblts, &size, &count);
@@ -1309,7 +1330,7 @@ nc_server_get_cpblts_version(const struct ly_ctx *ctx, LYS_VERSION version)
 
 unlock_error:
     /* HELLO UNLOCK */
-    pthread_rwlock_unlock(&server_opts.hello_lock);
+    nc_rwlock_unlock(&server_opts.hello_lock, __func__);
 
 error:
     free(cpblts);
