@@ -47,10 +47,15 @@ extern struct nc_server_opts server_opts;
 #define NC_READ_BUF_SIZE_STEP 512
 
 /**
- * Maximum size of a framing message chunk (spec max is 2^32 - 1).
+ * Maximum size of a written framing message chunk (spec max is 2^32 - 1).
  */
 #define NC_WRITE_CHUNK_SIZE_MAX 1024 * 10
 
+/**
+ * Maximum size of a read framing message chunk.
+ */
+#define NC_READ_CHUNK_SIZE_MAX 1024 * 1024 * 10
+
 /**
  * Sleep time in usec to wait between nc_recv_notif() calls.
  */
@@ -99,6 +104,54 @@ extern struct nc_server_opts server_opts;
  */
 #define NC_CH_CONNECT_TIMEOUT 500
 
+/**
+ * @brief Timeout in msec for acquiring the hello_lock
+ * (iterating through all YANG modules + building capability strings)
+ */
+#define NC_HELLO_LOCK_TIMEOUT 5000
+
+/**
+ * @brief Timeout in msec for acquiring the ch_threads_lock
+ * (simple array iteration and thread signaling operations)
+ */
+#define NC_CH_THREADS_LOCK_TIMEOUT 1000
+
+/**
+ * @brief Timeout in msec for acquiring the Call Home thread cond_lock
+ * (short critical sections used to signal thread termination and gate condition waits)
+ */
+#define NC_CH_COND_LOCK_TIMEOUT 1000
+
+/**
+ * @brief Timeout in msec for acquiring the certificate expiration notification lock
+ * (short critical sections for flag updates and condition signaling)
+ */
+#define NC_CERT_EXP_LOCK_TIMEOUT 1000
+
+/**
+ * @brief Timeout in msec for acquiring the config_lock
+ * (socket binding and Call Home client dispatching can involve network operations)
+ */
+#define NC_CONFIG_LOCK_TIMEOUT 10000
+
+/**
+ * @brief Timeout in msec for acquiring session's ch_lock
+ * (just simple session flag checks and updates)
+ */
+#define NC_SESSION_CH_LOCK_TIMEOUT 1000
+
+/**
+ * @brief Timeout in msec for acquiring the pollsession's lock
+ * (only O(n) array manipulation, where n is number of sessions (small usually))
+ */
+#define NC_PS_LOCK_TIMEOUT 1000
+
+/**
+ * @brief Timeout in msec for acquiring the notification status lock
+ * (short critical sections for incrementing/decrementing notification status)
+ */
+#define NC_SESSION_NTF_STATUS_LOCK_TIMEOUT 500
+
 /**
  * Number of sockets kept waiting to be accepted.
  */
@@ -133,6 +186,15 @@ enum nc_operation {
     NC_OP_REPLACE
 };
 
+/**
+ * @brief Enumeration of rwlock mode.
+ */
+enum nc_rwlock_mode {
+    NC_RWLOCK_NONE = 0,     /**< Lock not held */
+    NC_RWLOCK_READ,         /**< Read lock mode */
+    NC_RWLOCK_WRITE         /**< Write lock mode */
+};
+
 /**
  * Enumeration of key or certificate store type.
  */
@@ -613,6 +675,9 @@ struct nc_server_ch_thread_arg {
     void *ctx_cb_data;                                      /**< acq/rel cb data */
     int (*new_session_cb)(const char *client_name, struct nc_session *new_session, void *user_data);    /**< creating new session cb */
     void *new_session_cb_data;                              /**< new session cb data */
+    void (*new_session_fail_cb)(const char *client_name, const char *endpt_name, uint8_t max_attempts,
+            uint8_t cur_attempt, void *user_data);          /**< failed to create a new session cb */
+    void *new_session_fail_cb_data;                         /**< new session fail cb data */
 
     int thread_running;         /**< A boolean value that is truthy while the underlying Call Home thread is running */
     pthread_mutex_t cond_lock;  /**< Condition's lock used for signalling the thread to terminate */
@@ -736,6 +801,8 @@ struct nc_server_opts {
         void *ctx_cb_data;                                      /**< Data passed to the callbacks above. */
         nc_server_ch_new_session_cb new_session_cb;             /**< New session callback. */
         void *new_session_cb_data;                              /**< Data passed to the new_session_cb callback. */
+        nc_server_ch_new_session_fail_cb new_session_fail_cb;   /**< New session fail callback, */
+        void *new_session_fail_cb_data;                         /**< Data passed to the new_session_fail_cb callback. */
     } ch_dispatch_data;
 
     struct {
@@ -1083,48 +1150,54 @@ int nc_session_rpc_lock(struct nc_session *session, int timeout, const char *fun
 int nc_session_rpc_unlock(struct nc_session *session, int timeout, const char *func);
 
 /**
- * @brief Lock IO lock on a session.
+ * @brief Lock a pthread_rwlock with timeout support.
  *
- * @param[in] session Session to lock.
- * @param[in] timeout Timeout in msec to use.
- * @param[in] func Caller function for logging.
- * @return 1 on success;
+ * @param[in] rwlock Lock to be acquired.
+ * @param[in] mode Lock mode (NC_RWLOCK_READ for read lock, NC_RWLOCK_WRITE for write lock).
+ * @param[in] timeout Timeout in milliseconds:
+ *                    - Positive value: Wait up to this many milliseconds for the lock.
+ *                    - 0: Try to acquire the lock without waiting (trylock).
+ *                    - Negative value: Wait indefinitely for the lock.
+ * @param[in] func_name Caller function name for logging purposes.
+ * @return 1 on success (lock acquired);
  * @return 0 on timeout;
  * @return -1 on error.
  */
-int nc_session_io_lock(struct nc_session *session, int timeout, const char *func);
+int nc_rwlock_lock(pthread_rwlock_t *rwlock, enum nc_rwlock_mode mode, int timeout, const char *func_name);
 
 /**
- * @brief Unlock IO lock on a session.
+ * @brief Unlock a previously locked pthread_rwlock.
  *
- * @param[in] session Session to unlock.
- * @param[in] func Caller function for logging.
- * @return 1 on success;
- * @return -1 on error.
+ * This function unlocks a previously locked pthread_rwlock regardless of whether
+ * it was locked in read or write mode.
+ *
+ * @param[in] rwlock Lock to be unlocked.
+ * @param[in] func_name Caller function name for logging purposes.
  */
-int nc_session_io_unlock(struct nc_session *session, const char *func);
+void nc_rwlock_unlock(pthread_rwlock_t *rwlock, const char *func_name);
 
 /**
- * @brief Lock MSGS lock on a session.
+ * @brief Lock a pthread_mutex with timeout support.
  *
- * @param[in] session Session to lock.
- * @param[in,out] timeout Timeout in msec to use. If positive and on successful lock, is updated based on what was elapsed.
- * @param[in] func Caller function for logging.
- * @return 1 on success;
+ * @param[in] mutex Mutex to be acquired.
+ * @param[in] timeout Timeout in milliseconds:
+ *                    - Positive value: Wait up to this many milliseconds for the lock.
+ *                    - 0: Try to acquire the lock without waiting (trylock).
+ *                    - Negative value: Wait indefinitely for the lock.
+ * @param[in] func_name Caller function name for logging purposes.
+ * @return 1 on success (lock acquired);
  * @return 0 on timeout;
  * @return -1 on error.
  */
-int nc_session_client_msgs_lock(struct nc_session *session, int *timeout, const char *func);
+int nc_mutex_lock(pthread_mutex_t *mutex, int timeout, const char *func_name);
 
 /**
- * @brief Unlock MSGS lock on a session.
+ * @brief Unlock a previously locked pthread_mutex.
  *
- * @param[in] session Session to unlock.
- * @param[in] func Caller function for logging.
- * @return 1 on success;
- * @return -1 on error.
+ * @param[in] mutex Mutex to be unlocked.
+ * @param[in] func_name Caller function name for logging purposes.
  */
-int nc_session_client_msgs_unlock(struct nc_session *session, const char *func);
+void nc_mutex_unlock(pthread_mutex_t *mutex, const char *func_name);
 
 int nc_ps_lock(struct nc_pollsession *ps, uint8_t *id, const char *func);
 
